@@ -54,12 +54,12 @@ class XmlHandler(ContentHandler):
         if element == "record":
             self._record = Record()
         elif element == "controlfield":
-            tag = attrs.getValue((None, u"tag"))
+            tag = attrs.getValue((None, "tag"))
             self._field = Field(tag)
         elif element == "datafield":
-            tag = attrs.getValue((None, u"tag"))
-            ind1 = attrs.get((None, u"ind1"), u" ")
-            ind2 = attrs.get((None, u"ind2"), u" ")
+            tag = attrs.getValue((None, "tag"))
+            ind1 = attrs.get((None, "ind1"), " ")
+            ind2 = attrs.get((None, "ind2"), " ")
             self._field = Field(tag, [ind1, ind2])
         elif element == "subfield":
             self._subfield_code = attrs[(None, "code")]
@@ -71,9 +71,9 @@ class XmlHandler(ContentHandler):
 
         element = name[1]
         if self.normalize_form is not None:
-            text = unicodedata.normalize(self.normalize_form, u"".join(self._text))
+            text = unicodedata.normalize(self.normalize_form, "".join(self._text))
         else:
-            text = u"".join(self._text)
+            text = "".join(self._text)
 
         if element == "record":
             self.process_record(self._record)
